using Avalonia.Controls;
using Avalonia.Interactivity;
using Avalonia.Media;
using Avalonia.Media.Imaging;
using Avalonia.Utilities;
using demoModels;
using Microsoft.EntityFrameworkCore;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel.DataAnnotations;
using System.Linq;

namespace demo
{
    public partial class MainWindow : Window
    {
        ObservableCollection<ProductPresenter> products =  new ObservableCollection<ProductPresenter>();  
        List<ProductPresenter> dataSourceProducts;  
        ObservableCollection<string> manufactures;  
        public MainWindow()
        {
            InitializeComponent();
            using var context = new ProductsTradeContext();
            dataSourceProducts = context.Products.Include(it => it.Manufacturer).Select(product => new ProductPresenter
            {
                Id = product.Id,
                Mainimagepath = product.Mainimagepath,
                Isactive = product.Isactive,
                Cost = product.Cost,
                Title = product.Title,
                Description = product.Description,
                Manufacturer = product.Manufacturer,
            }).ToList();
            var dataSourceManafacture = context.Manufacturers.Select(it => it.Name).ToList();
            manufactures = new ObservableCollection<string>(dataSourceManafacture);
            manufactures.Insert(0," ");
            ProductBox.ItemsSource = products;
            ManufactureBox.ItemsSource = manufactures;
            ManufactureBox.SelectedIndex = 0;
            DisplayProducts();
        }

        private void DisplayProducts()
        {
            var temp = dataSourceProducts;
            products.Clear();
            if (ManufactureBox.SelectedIndex != 0)
            {
                temp = temp.Where(it => it.ManufacatureName.Contains(ManufactureBox.SelectedItem.ToString())).ToList();
            }
            if (!string.IsNullOrEmpty(SearchBox.Text))
            {
                var searchWord = SearchBox.Text.ToLower();  
                temp = temp.Where(it => IsContains(it.Title, it.Description, searchWord)).ToList();
            }
            switch (SortBox.SelectedIndex) {

                case 1: temp = temp.OrderBy(temp => temp.Cost).ToList(); break;
                case 2: temp = temp.OrderByDescending(temp => temp.Cost).ToList(); break;
                default: break;
            }
            foreach (var item in temp)
            {
                products.Add(item);
            }
            CountTextBlock.Text = $"{temp.Count}\\{dataSourceProducts.Count}";


        }

        public bool IsContains(string title, string? description, string searchWord) {
            string desc = string.Empty;
            if(description != null) desc = description;
            string message = (title + desc).ToLower();
            searchWord = searchWord.ToLower();
            var result = message.Contains(searchWord);
            return message.Contains(searchWord);
        }
        public void SearchBoxChanging(object sender, TextChangingEventArgs eventArgs)
        {
            DisplayProducts();
        }

        public void SortBox_SelectionChanged(object sender, SelectionChangedEventArgs eventArgs) {
            DisplayProducts();
        }
        public void ManufactureBox_SelectionChanged(object sender, SelectionChangedEventArgs eventArgs)
        {
            DisplayProducts();
        }
        public void ProductItemClick(object sender, RoutedEventArgs e)
        {
            using var context = new ProductsTradeContext();
            var product = ProductBox.SelectedItem as ProductPresenter;
            if (product == null) return;
            var removeProduct = context.Products.Include(it => it.Productsales).First(it => it.Id == product.Id);
            if(removeProduct == null) return;
            if(removeProduct.Productsales.Count() > 0) return;
            context.Products.Remove(removeProduct);
            if (context.SaveChanges() > 0) products.Remove(product);
        }
        public async void AddNewProduct(object sender, RoutedEventArgs e)
        {
           using ProductsTradeContext context = new ProductsTradeContext();   
           var newProduct =  await new ProductWindow().ShowDialog<ProductPresenter>(this);

           context.Products.Add(newProduct);
            if (context.SaveChanges() > 0) {
                newProduct.Manufacturer = context.Manufacturers.Find(newProduct.Manufacturerid);
                dataSourceProducts.Add(newProduct);
            }
        }
    }
    public class ProductPresenter() : Product{
        public string ManufacatureName { get => Manufacturer.Name; }
        Bitmap? Image { get
            {
                try
                {
                    return new Bitmap(Mainimagepath);
                }
                catch
                {
                    return null;
                }
            } }
        public IBrush Color { get {
                return Isactive?  Brushes.White : Brushes.Gray;
            } }
    }
}